////////////////////////////////////////////////////////////////////////
//                                                                    //
// MicrOTP: A tool for computing one time passwords for S/Key logins. //
//                                                                    //
// (C) 2008 Tom Bramer                                                //
//                                                                    //
// This program is free software; you can redistribute it and/or      //
// modify it under the terms of the GNU General Public License        //
// as published by the Free Software Foundation; either version 2     //
// of the License, or (at your option) any later version.             //
//                                                                    //
// This program is distributed in the hope that it will be useful,    //
// but WITHOUT ANY WARRANTY; without even the implied warranty of     //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      //
// GNU General Public License for more details.                       //
//                                                                    //
// You should have received a copy of the GNU General Public License  //
// along with this program; if not, write to the Free Software        //
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA      //
// 02110-1301, USA.                                                   //
//                                                                    //
////////////////////////////////////////////////////////////////////////

// OTPDictionary.cpp: implementation of the COTPDictionary class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "otp3.h"
#include "OTPDictionary.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// The actual word dictionary
//////////////////////////////////////////////////////////////////////

#define DICTIONARY_SIZE   2048

static TCHAR* words[] = {
    _T("A"),     _T("ABE"),   _T("ACE"),   _T("ACT"),   _T("AD"),    _T("ADA"),   _T("ADD"),
    _T("AGO"),   _T("AID"),   _T("AIM"),   _T("AIR"),   _T("ALL"),   _T("ALP"),   _T("AM"),    _T("AMY"),
    _T("AN"),    _T("ANA"),   _T("AND"),   _T("ANN"),   _T("ANT"),   _T("ANY"),   _T("APE"),   _T("APS"),
    _T("APT"),   _T("ARC"),   _T("ARE"),   _T("ARK"),   _T("ARM"),   _T("ART"),   _T("AS"),    _T("ASH"),
    _T("ASK"),   _T("AT"),    _T("ATE"),   _T("AUG"),   _T("AUK"),   _T("AVE"),   _T("AWE"),   _T("AWK"),
    _T("AWL"),   _T("AWN"),   _T("AX"),    _T("AYE"),   _T("BAD"),   _T("BAG"),   _T("BAH"),   _T("BAM"),
    _T("BAN"),   _T("BAR"),   _T("BAT"),   _T("BAY"),   _T("BE"),    _T("BED"),   _T("BEE"),   _T("BEG"),
    _T("BEN"),   _T("BET"),   _T("BEY"),   _T("BIB"),   _T("BID"),   _T("BIG"),   _T("BIN"),   _T("BIT"),
    _T("BOB"),   _T("BOG"),   _T("BON"),   _T("BOO"),   _T("BOP"),   _T("BOW"),   _T("BOY"),   _T("BUB"),
    _T("BUD"),   _T("BUG"),   _T("BUM"),   _T("BUN"),   _T("BUS"),   _T("BUT"),   _T("BUY"),   _T("BY"),
    _T("BYE"),   _T("CAB"),   _T("CAL"),   _T("CAM"),   _T("CAN"),   _T("CAP"),   _T("CAR"),   _T("CAT"),
    _T("CAW"),   _T("COD"),   _T("COG"),   _T("COL"),   _T("CON"),   _T("COO"),   _T("COP"),   _T("COT"),
    _T("COW"),   _T("COY"),   _T("CRY"),   _T("CUB"),   _T("CUE"),   _T("CUP"),   _T("CUR"),   _T("CUT"),
    _T("DAB"),   _T("DAD"),   _T("DAM"),   _T("DAN"),   _T("DAR"),   _T("DAY"),   _T("DEE"),   _T("DEL"),
    _T("DEN"),   _T("DES"),   _T("DEW"),   _T("DID"),   _T("DIE"),   _T("DIG"),   _T("DIN"),   _T("DIP"),
    _T("DO"),    _T("DOE"),   _T("DOG"),   _T("DON"),   _T("DOT"),   _T("DOW"),   _T("DRY"),   _T("DUB"),
    _T("DUD"),   _T("DUE"),   _T("DUG"),   _T("DUN"),   _T("EAR"),   _T("EAT"),   _T("ED"),    _T("EEL"),
    _T("EGG"),   _T("EGO"),   _T("ELI"),   _T("ELK"),   _T("ELM"),   _T("ELY"),   _T("EM"),    _T("END"),
    _T("EST"),   _T("ETC"),   _T("EVA"),   _T("EVE"),   _T("EWE"),   _T("EYE"),   _T("FAD"),   _T("FAN"),
    _T("FAR"),   _T("FAT"),   _T("FAY"),   _T("FED"),   _T("FEE"),   _T("FEW"),   _T("FIB"),   _T("FIG"),
    _T("FIN"),   _T("FIR"),   _T("FIT"),   _T("FLO"),   _T("FLY"),   _T("FOE"),   _T("FOG"),   _T("FOR"),
    _T("FRY"),   _T("FUM"),   _T("FUN"),   _T("FUR"),   _T("GAB"),   _T("GAD"),   _T("GAG"),   _T("GAL"),
    _T("GAM"),   _T("GAP"),   _T("GAS"),   _T("GAY"),   _T("GEE"),   _T("GEL"),   _T("GEM"),   _T("GET"),
    _T("GIG"),   _T("GIL"),   _T("GIN"),   _T("GO"),    _T("GOT"),   _T("GUM"),   _T("GUN"),   _T("GUS"),
    _T("GUT"),   _T("GUY"),   _T("GYM"),   _T("GYP"),   _T("HA"),    _T("HAD"),   _T("HAL"),   _T("HAM"),
    _T("HAN"),   _T("HAP"),   _T("HAS"),   _T("HAT"),   _T("HAW"),   _T("HAY"),   _T("HE"),    _T("HEM"),
    _T("HEN"),   _T("HER"),   _T("HEW"),   _T("HEY"),   _T("HI"),    _T("HID"),   _T("HIM"),   _T("HIP"),
    _T("HIS"),   _T("HIT"),   _T("HO"),    _T("HOB"),   _T("HOC"),   _T("HOE"),   _T("HOG"),   _T("HOP"),
    _T("HOT"),   _T("HOW"),   _T("HUB"),   _T("HUE"),   _T("HUG"),   _T("HUH"),   _T("HUM"),   _T("HUT"),
    _T("I"),     _T("ICY"),   _T("IDA"),   _T("IF"),    _T("IKE"),   _T("ILL"),   _T("INK"),   _T("INN"),
    _T("IO"),    _T("ION"),   _T("IQ"),    _T("IRA"),   _T("IRE"),   _T("IRK"),   _T("IS"),    _T("IT"),
    _T("ITS"),   _T("IVY"),   _T("JAB"),   _T("JAG"),   _T("JAM"),   _T("JAN"),   _T("JAR"),   _T("JAW"),
    _T("JAY"),   _T("JET"),   _T("JIG"),   _T("JIM"),   _T("JO"),    _T("JOB"),   _T("JOE"),   _T("JOG"),
    _T("JOT"),   _T("JOY"),   _T("JUG"),   _T("JUT"),   _T("KAY"),   _T("KEG"),   _T("KEN"),   _T("KEY"),
    _T("KID"),   _T("KIM"),   _T("KIN"),   _T("KIT"),   _T("LA"),    _T("LAB"),   _T("LAC"),   _T("LAD"),
    _T("LAG"),   _T("LAM"),   _T("LAP"),   _T("LAW"),   _T("LAY"),   _T("LEA"),   _T("LED"),   _T("LEE"),
    _T("LEG"),   _T("LEN"),   _T("LEO"),   _T("LET"),   _T("LEW"),   _T("LID"),   _T("LIE"),   _T("LIN"),
    _T("LIP"),   _T("LIT"),   _T("LO"),    _T("LOB"),   _T("LOG"),   _T("LOP"),   _T("LOS"),   _T("LOT"),
    _T("LOU"),   _T("LOW"),   _T("LOY"),   _T("LUG"),   _T("LYE"),   _T("MA"),    _T("MAC"),   _T("MAD"),
    _T("MAE"),   _T("MAN"),   _T("MAO"),   _T("MAP"),   _T("MAT"),   _T("MAW"),   _T("MAY"),   _T("ME"),
    _T("MEG"),   _T("MEL"),   _T("MEN"),   _T("MET"),   _T("MEW"),   _T("MID"),   _T("MIN"),   _T("MIT"),
    _T("MOB"),   _T("MOD"),   _T("MOE"),   _T("MOO"),   _T("MOP"),   _T("MOS"),   _T("MOT"),   _T("MOW"),
    _T("MUD"),   _T("MUG"),   _T("MUM"),   _T("MY"),    _T("NAB"),   _T("NAG"),   _T("NAN"),   _T("NAP"),
    _T("NAT"),   _T("NAY"),   _T("NE"),    _T("NED"),   _T("NEE"),   _T("NET"),   _T("NEW"),   _T("NIB"),
    _T("NIL"),   _T("NIP"),   _T("NIT"),   _T("NO"),    _T("NOB"),   _T("NOD"),   _T("NON"),   _T("NOR"),
    _T("NOT"),   _T("NOV"),   _T("NOW"),   _T("NU"),    _T("NUN"),   _T("NUT"),   _T("O"),     _T("OAF"),
    _T("OAK"),   _T("OAR"),   _T("OAT"),   _T("ODD"),   _T("ODE"),   _T("OF"),    _T("OFF"),   _T("OFT"),
    _T("OH"),    _T("OIL"),   _T("OK"),    _T("OLD"),   _T("ON"),    _T("ONE"),   _T("OR"),    _T("ORB"),
    _T("ORE"),   _T("ORR"),   _T("OS"),    _T("OTT"),   _T("OUR"),   _T("OUT"),   _T("OVA"),   _T("OW"),
    _T("OWE"),   _T("OWL"),   _T("OWN"),   _T("OX"),    _T("PA"),    _T("PAD"),   _T("PAL"),   _T("PAM"),
    _T("PAN"),   _T("PAP"),   _T("PAR"),   _T("PAT"),   _T("PAW"),   _T("PAY"),   _T("PEA"),   _T("PEG"),
    _T("PEN"),   _T("PEP"),   _T("PER"),   _T("PET"),   _T("PEW"),   _T("PHI"),   _T("PI"),    _T("PIE"),
    _T("PIN"),   _T("PIT"),   _T("PLY"),   _T("PO"),    _T("POD"),   _T("POE"),   _T("POP"),   _T("POT"),
    _T("POW"),   _T("PRO"),   _T("PRY"),   _T("PUB"),   _T("PUG"),   _T("PUN"),   _T("PUP"),   _T("PUT"),
    _T("QUO"),   _T("RAG"),   _T("RAM"),   _T("RAN"),   _T("RAP"),   _T("RAT"),   _T("RAW"),   _T("RAY"),
    _T("REB"),   _T("RED"),   _T("REP"),   _T("RET"),   _T("RIB"),   _T("RID"),   _T("RIG"),   _T("RIM"),
    _T("RIO"),   _T("RIP"),   _T("ROB"),   _T("ROD"),   _T("ROE"),   _T("RON"),   _T("ROT"),   _T("ROW"),
    _T("ROY"),   _T("RUB"),   _T("RUE"),   _T("RUG"),   _T("RUM"),   _T("RUN"),   _T("RYE"),   _T("SAC"),
    _T("SAD"),   _T("SAG"),   _T("SAL"),   _T("SAM"),   _T("SAN"),   _T("SAP"),   _T("SAT"),   _T("SAW"),
    _T("SAY"),   _T("SEA"),   _T("SEC"),   _T("SEE"),   _T("SEN"),   _T("SET"),   _T("SEW"),   _T("SHE"),
    _T("SHY"),   _T("SIN"),   _T("SIP"),   _T("SIR"),   _T("SIS"),   _T("SIT"),   _T("SKI"),   _T("SKY"),
    _T("SLY"),   _T("SO"),    _T("SOB"),   _T("SOD"),   _T("SON"),   _T("SOP"),   _T("SOW"),   _T("SOY"),
    _T("SPA"),   _T("SPY"),   _T("SUB"),   _T("SUD"),   _T("SUE"),   _T("SUM"),   _T("SUN"),   _T("SUP"),
    _T("TAB"),   _T("TAD"),   _T("TAG"),   _T("TAN"),   _T("TAP"),   _T("TAR"),   _T("TEA"),   _T("TED"),
    _T("TEE"),   _T("TEN"),   _T("THE"),   _T("THY"),   _T("TIC"),   _T("TIE"),   _T("TIM"),   _T("TIN"),
    _T("TIP"),   _T("TO"),    _T("TOE"),   _T("TOG"),   _T("TOM"),   _T("TON"),   _T("TOO"),   _T("TOP"),
    _T("TOW"),   _T("TOY"),   _T("TRY"),   _T("TUB"),   _T("TUG"),   _T("TUM"),   _T("TUN"),   _T("TWO"),
    _T("UN"),    _T("UP"),    _T("US"),    _T("USE"),   _T("VAN"),   _T("VAT"),   _T("VET"),   _T("VIE"),
    _T("WAD"),   _T("WAG"),   _T("WAR"),   _T("WAS"),   _T("WAY"),   _T("WE"),    _T("WEB"),   _T("WED"),
    _T("WEE"),   _T("WET"),   _T("WHO"),   _T("WHY"),   _T("WIN"),   _T("WIT"),   _T("WOK"),   _T("WON"),
    _T("WOO"),   _T("WOW"),   _T("WRY"),   _T("WU"),    _T("YAM"),   _T("YAP"),   _T("YAW"),   _T("YE"),
    _T("YEA"),   _T("YES"),   _T("YET"),   _T("YOU"),   _T("ABED"),  _T("ABEL"),  _T("ABET"),  _T("ABLE"),
    _T("ABUT"),  _T("ACHE"),  _T("ACID"),  _T("ACME"),  _T("ACRE"),  _T("ACTA"),  _T("ACTS"),  _T("ADAM"),
    _T("ADDS"),  _T("ADEN"),  _T("AFAR"),  _T("AFRO"),  _T("AGEE"),  _T("AHEM"),  _T("AHOY"),  _T("AIDA"),
    _T("AIDE"),  _T("AIDS"),  _T("AIRY"),  _T("AJAR"),  _T("AKIN"),  _T("ALAN"),  _T("ALEC"),  _T("ALGA"),
    _T("ALIA"),  _T("ALLY"),  _T("ALMA"),  _T("ALOE"),  _T("ALSO"),  _T("ALTO"),  _T("ALUM"),  _T("ALVA"),
    _T("AMEN"),  _T("AMES"),  _T("AMID"),  _T("AMMO"),  _T("AMOK"),  _T("AMOS"),  _T("AMRA"),  _T("ANDY"),
    _T("ANEW"),  _T("ANNA"),  _T("ANNE"),  _T("ANTE"),  _T("ANTI"),  _T("AQUA"),  _T("ARAB"),  _T("ARCH"),
    _T("AREA"),  _T("ARGO"),  _T("ARID"),  _T("ARMY"),  _T("ARTS"),  _T("ARTY"),  _T("ASIA"),  _T("ASKS"),
    _T("ATOM"),  _T("AUNT"),  _T("AURA"),  _T("AUTO"),  _T("AVER"),  _T("AVID"),  _T("AVIS"),  _T("AVON"),
    _T("AVOW"),  _T("AWAY"),  _T("AWRY"),  _T("BABE"),  _T("BABY"),  _T("BACH"),  _T("BACK"),  _T("BADE"),
    _T("BAIL"),  _T("BAIT"),  _T("BAKE"),  _T("BALD"),  _T("BALE"),  _T("BALI"),  _T("BALK"),  _T("BALL"),
    _T("BALM"),  _T("BAND"),  _T("BANE"),  _T("BANG"),  _T("BANK"),  _T("BARB"),  _T("BARD"),  _T("BARE"),
    _T("BARK"),  _T("BARN"),  _T("BARR"),  _T("BASE"),  _T("BASH"),  _T("BASK"),  _T("BASS"),  _T("BATE"),
    _T("BATH"),  _T("BAWD"),  _T("BAWL"),  _T("BEAD"),  _T("BEAK"),  _T("BEAM"),  _T("BEAN"),  _T("BEAR"),
    _T("BEAT"),  _T("BEAU"),  _T("BECK"),  _T("BEEF"),  _T("BEEN"),  _T("BEER"),  _T("BEET"),  _T("BELA"),
    _T("BELL"),  _T("BELT"),  _T("BEND"),  _T("BENT"),  _T("BERG"),  _T("BERN"),  _T("BERT"),  _T("BESS"),
    _T("BEST"),  _T("BETA"),  _T("BETH"),  _T("BHOY"),  _T("BIAS"),  _T("BIDE"),  _T("BIEN"),  _T("BILE"),
    _T("BILK"),  _T("BILL"),  _T("BIND"),  _T("BING"),  _T("BIRD"),  _T("BITE"),  _T("BITS"),  _T("BLAB"),
    _T("BLAT"),  _T("BLED"),  _T("BLEW"),  _T("BLOB"),  _T("BLOC"),  _T("BLOT"),  _T("BLOW"),  _T("BLUE"),
    _T("BLUM"),  _T("BLUR"),  _T("BOAR"),  _T("BOAT"),  _T("BOCA"),  _T("BOCK"),  _T("BODE"),  _T("BODY"),
    _T("BOGY"),  _T("BOHR"),  _T("BOIL"),  _T("BOLD"),  _T("BOLO"),  _T("BOLT"),  _T("BOMB"),  _T("BONA"),
    _T("BOND"),  _T("BONE"),  _T("BONG"),  _T("BONN"),  _T("BONY"),  _T("BOOK"),  _T("BOOM"),  _T("BOON"),
    _T("BOOT"),  _T("BORE"),  _T("BORG"),  _T("BORN"),  _T("BOSE"),  _T("BOSS"),  _T("BOTH"),  _T("BOUT"),
    _T("BOWL"),  _T("BOYD"),  _T("BRAD"),  _T("BRAE"),  _T("BRAG"),  _T("BRAN"),  _T("BRAY"),  _T("BRED"),
    _T("BREW"),  _T("BRIG"),  _T("BRIM"),  _T("BROW"),  _T("BUCK"),  _T("BUDD"),  _T("BUFF"),  _T("BULB"),
    _T("BULK"),  _T("BULL"),  _T("BUNK"),  _T("BUNT"),  _T("BUOY"),  _T("BURG"),  _T("BURL"),  _T("BURN"),
    _T("BURR"),  _T("BURT"),  _T("BURY"),  _T("BUSH"),  _T("BUSS"),  _T("BUST"),  _T("BUSY"),  _T("BYTE"),
    _T("CADY"),  _T("CAFE"),  _T("CAGE"),  _T("CAIN"),  _T("CAKE"),  _T("CALF"),  _T("CALL"),  _T("CALM"),
    _T("CAME"),  _T("CANE"),  _T("CANT"),  _T("CARD"),  _T("CARE"),  _T("CARL"),  _T("CARR"),  _T("CART"),
    _T("CASE"),  _T("CASH"),  _T("CASK"),  _T("CAST"),  _T("CAVE"),  _T("CEIL"),  _T("CELL"),  _T("CENT"),
    _T("CERN"),  _T("CHAD"),  _T("CHAR"),  _T("CHAT"),  _T("CHAW"),  _T("CHEF"),  _T("CHEN"),  _T("CHEW"),
    _T("CHIC"),  _T("CHIN"),  _T("CHOU"),  _T("CHOW"),  _T("CHUB"),  _T("CHUG"),  _T("CHUM"),  _T("CITE"),
    _T("CITY"),  _T("CLAD"),  _T("CLAM"),  _T("CLAN"),  _T("CLAW"),  _T("CLAY"),  _T("CLOD"),  _T("CLOG"),
    _T("CLOT"),  _T("CLUB"),  _T("CLUE"),  _T("COAL"),  _T("COAT"),  _T("COCA"),  _T("COCK"),  _T("COCO"),
    _T("CODA"),  _T("CODE"),  _T("CODY"),  _T("COED"),  _T("COIL"),  _T("COIN"),  _T("COKE"),  _T("COLA"),
    _T("COLD"),  _T("COLT"),  _T("COMA"),  _T("COMB"),  _T("COME"),  _T("COOK"),  _T("COOL"),  _T("COON"),
    _T("COOT"),  _T("CORD"),  _T("CORE"),  _T("CORK"),  _T("CORN"),  _T("COST"),  _T("COVE"),  _T("COWL"),
    _T("CRAB"),  _T("CRAG"),  _T("CRAM"),  _T("CRAY"),  _T("CREW"),  _T("CRIB"),  _T("CROW"),  _T("CRUD"),
    _T("CUBA"),  _T("CUBE"),  _T("CUFF"),  _T("CULL"),  _T("CULT"),  _T("CUNY"),  _T("CURB"),  _T("CURD"),
    _T("CURE"),  _T("CURL"),  _T("CURT"),  _T("CUTS"),  _T("DADE"),  _T("DALE"),  _T("DAME"),  _T("DANA"),
    _T("DANE"),  _T("DANG"),  _T("DANK"),  _T("DARE"),  _T("DARK"),  _T("DARN"),  _T("DART"),  _T("DASH"),
    _T("DATA"),  _T("DATE"),  _T("DAVE"),  _T("DAVY"),  _T("DAWN"),  _T("DAYS"),  _T("DEAD"),  _T("DEAF"),
    _T("DEAL"),  _T("DEAN"),  _T("DEAR"),  _T("DEBT"),  _T("DECK"),  _T("DEED"),  _T("DEEM"),  _T("DEER"),
    _T("DEFT"),  _T("DEFY"),  _T("DELL"),  _T("DENT"),  _T("DENY"),  _T("DESK"),  _T("DIAL"),  _T("DICE"),
    _T("DIED"),  _T("DIET"),  _T("DIME"),  _T("DINE"),  _T("DING"),  _T("DINT"),  _T("DIRE"),  _T("DIRT"),
    _T("DISC"),  _T("DISH"),  _T("DISK"),  _T("DIVE"),  _T("DOCK"),  _T("DOES"),  _T("DOLE"),  _T("DOLL"),
    _T("DOLT"),  _T("DOME"),  _T("DONE"),  _T("DOOM"),  _T("DOOR"),  _T("DORA"),  _T("DOSE"),  _T("DOTE"),
    _T("DOUG"),  _T("DOUR"),  _T("DOVE"),  _T("DOWN"),  _T("DRAB"),  _T("DRAG"),  _T("DRAM"),  _T("DRAW"),
    _T("DREW"),  _T("DRUB"),  _T("DRUG"),  _T("DRUM"),  _T("DUAL"),  _T("DUCK"),  _T("DUCT"),  _T("DUEL"),
    _T("DUET"),  _T("DUKE"),  _T("DULL"),  _T("DUMB"),  _T("DUNE"),  _T("DUNK"),  _T("DUSK"),  _T("DUST"),
    _T("DUTY"),  _T("EACH"),  _T("EARL"),  _T("EARN"),  _T("EASE"),  _T("EAST"),  _T("EASY"),  _T("EBEN"),
    _T("ECHO"),  _T("EDDY"),  _T("EDEN"),  _T("EDGE"),  _T("EDGY"),  _T("EDIT"),  _T("EDNA"),  _T("EGAN"),
    _T("ELAN"),  _T("ELBA"),  _T("ELLA"),  _T("ELSE"),  _T("EMIL"),  _T("EMIT"),  _T("EMMA"),  _T("ENDS"),
    _T("ERIC"),  _T("EROS"),  _T("EVEN"),  _T("EVER"),  _T("EVIL"),  _T("EYED"),  _T("FACE"),  _T("FACT"),
    _T("FADE"),  _T("FAIL"),  _T("FAIN"),  _T("FAIR"),  _T("FAKE"),  _T("FALL"),  _T("FAME"),  _T("FANG"),
    _T("FARM"),  _T("FAST"),  _T("FATE"),  _T("FAWN"),  _T("FEAR"),  _T("FEAT"),  _T("FEED"),  _T("FEEL"),
    _T("FEET"),  _T("FELL"),  _T("FELT"),  _T("FEND"),  _T("FERN"),  _T("FEST"),  _T("FEUD"),  _T("FIEF"),
    _T("FIGS"),  _T("FILE"),  _T("FILL"),  _T("FILM"),  _T("FIND"),  _T("FINE"),  _T("FINK"),  _T("FIRE"),
    _T("FIRM"),  _T("FISH"),  _T("FISK"),  _T("FIST"),  _T("FITS"),  _T("FIVE"),  _T("FLAG"),  _T("FLAK"),
    _T("FLAM"),  _T("FLAT"),  _T("FLAW"),  _T("FLEA"),  _T("FLED"),  _T("FLEW"),  _T("FLIT"),  _T("FLOC"),
    _T("FLOG"),  _T("FLOW"),  _T("FLUB"),  _T("FLUE"),  _T("FOAL"),  _T("FOAM"),  _T("FOGY"),  _T("FOIL"),
    _T("FOLD"),  _T("FOLK"),  _T("FOND"),  _T("FONT"),  _T("FOOD"),  _T("FOOL"),  _T("FOOT"),  _T("FORD"),
    _T("FORE"),  _T("FORK"),  _T("FORM"),  _T("FORT"),  _T("FOSS"),  _T("FOUL"),  _T("FOUR"),  _T("FOWL"),
    _T("FRAU"),  _T("FRAY"),  _T("FRED"),  _T("FREE"),  _T("FRET"),  _T("FREY"),  _T("FROG"),  _T("FROM"),
    _T("FUEL"),  _T("FULL"),  _T("FUME"),  _T("FUND"),  _T("FUNK"),  _T("FURY"),  _T("FUSE"),  _T("FUSS"),
    _T("GAFF"),  _T("GAGE"),  _T("GAIL"),  _T("GAIN"),  _T("GAIT"),  _T("GALA"),  _T("GALE"),  _T("GALL"),
    _T("GALT"),  _T("GAME"),  _T("GANG"),  _T("GARB"),  _T("GARY"),  _T("GASH"),  _T("GATE"),  _T("GAUL"),
    _T("GAUR"),  _T("GAVE"),  _T("GAWK"),  _T("GEAR"),  _T("GELD"),  _T("GENE"),  _T("GENT"),  _T("GERM"),
    _T("GETS"),  _T("GIBE"),  _T("GIFT"),  _T("GILD"),  _T("GILL"),  _T("GILT"),  _T("GINA"),  _T("GIRD"),
    _T("GIRL"),  _T("GIST"),  _T("GIVE"),  _T("GLAD"),  _T("GLEE"),  _T("GLEN"),  _T("GLIB"),  _T("GLOB"),
    _T("GLOM"),  _T("GLOW"),  _T("GLUE"),  _T("GLUM"),  _T("GLUT"),  _T("GOAD"),  _T("GOAL"),  _T("GOAT"),
    _T("GOER"),  _T("GOES"),  _T("GOLD"),  _T("GOLF"),  _T("GONE"),  _T("GONG"),  _T("GOOD"),  _T("GOOF"),
    _T("GORE"),  _T("GORY"),  _T("GOSH"),  _T("GOUT"),  _T("GOWN"),  _T("GRAB"),  _T("GRAD"),  _T("GRAY"),
    _T("GREG"),  _T("GREW"),  _T("GREY"),  _T("GRID"),  _T("GRIM"),  _T("GRIN"),  _T("GRIT"),  _T("GROW"),
    _T("GRUB"),  _T("GULF"),  _T("GULL"),  _T("GUNK"),  _T("GURU"),  _T("GUSH"),  _T("GUST"),  _T("GWEN"),
    _T("GWYN"),  _T("HAAG"),  _T("HAAS"),  _T("HACK"),  _T("HAIL"),  _T("HAIR"),  _T("HALE"),  _T("HALF"),
    _T("HALL"),  _T("HALO"),  _T("HALT"),  _T("HAND"),  _T("HANG"),  _T("HANK"),  _T("HANS"),  _T("HARD"),
    _T("HARK"),  _T("HARM"),  _T("HART"),  _T("HASH"),  _T("HAST"),  _T("HATE"),  _T("HATH"),  _T("HAUL"),
    _T("HAVE"),  _T("HAWK"),  _T("HAYS"),  _T("HEAD"),  _T("HEAL"),  _T("HEAR"),  _T("HEAT"),  _T("HEBE"),
    _T("HECK"),  _T("HEED"),  _T("HEEL"),  _T("HEFT"),  _T("HELD"),  _T("HELL"),  _T("HELM"),  _T("HERB"),
    _T("HERD"),  _T("HERE"),  _T("HERO"),  _T("HERS"),  _T("HESS"),  _T("HEWN"),  _T("HICK"),  _T("HIDE"),
    _T("HIGH"),  _T("HIKE"),  _T("HILL"),  _T("HILT"),  _T("HIND"),  _T("HINT"),  _T("HIRE"),  _T("HISS"),
    _T("HIVE"),  _T("HOBO"),  _T("HOCK"),  _T("HOFF"),  _T("HOLD"),  _T("HOLE"),  _T("HOLM"),  _T("HOLT"),
    _T("HOME"),  _T("HONE"),  _T("HONK"),  _T("HOOD"),  _T("HOOF"),  _T("HOOK"),  _T("HOOT"),  _T("HORN"),
    _T("HOSE"),  _T("HOST"),  _T("HOUR"),  _T("HOVE"),  _T("HOWE"),  _T("HOWL"),  _T("HOYT"),  _T("HUCK"),
    _T("HUED"),  _T("HUFF"),  _T("HUGE"),  _T("HUGH"),  _T("HUGO"),  _T("HULK"),  _T("HULL"),  _T("HUNK"),
    _T("HUNT"),  _T("HURD"),  _T("HURL"),  _T("HURT"),  _T("HUSH"),  _T("HYDE"),  _T("HYMN"),  _T("IBIS"),
    _T("ICON"),  _T("IDEA"),  _T("IDLE"),  _T("IFFY"),  _T("INCA"),  _T("INCH"),  _T("INTO"),  _T("IONS"),
    _T("IOTA"),  _T("IOWA"),  _T("IRIS"),  _T("IRMA"),  _T("IRON"),  _T("ISLE"),  _T("ITCH"),  _T("ITEM"),
    _T("IVAN"),  _T("JACK"),  _T("JADE"),  _T("JAIL"),  _T("JAKE"),  _T("JANE"),  _T("JAVA"),  _T("JEAN"),
    _T("JEFF"),  _T("JERK"),  _T("JESS"),  _T("JEST"),  _T("JIBE"),  _T("JILL"),  _T("JILT"),  _T("JIVE"),
    _T("JOAN"),  _T("JOBS"),  _T("JOCK"),  _T("JOEL"),  _T("JOEY"),  _T("JOHN"),  _T("JOIN"),  _T("JOKE"),
    _T("JOLT"),  _T("JOVE"),  _T("JUDD"),  _T("JUDE"),  _T("JUDO"),  _T("JUDY"),  _T("JUJU"),  _T("JUKE"),
    _T("JULY"),  _T("JUNE"),  _T("JUNK"),  _T("JUNO"),  _T("JURY"),  _T("JUST"),  _T("JUTE"),  _T("KAHN"),
    _T("KALE"),  _T("KANE"),  _T("KANT"),  _T("KARL"),  _T("KATE"),  _T("KEEL"),  _T("KEEN"),  _T("KENO"),
    _T("KENT"),  _T("KERN"),  _T("KERR"),  _T("KEYS"),  _T("KICK"),  _T("KILL"),  _T("KIND"),  _T("KING"),
    _T("KIRK"),  _T("KISS"),  _T("KITE"),  _T("KLAN"),  _T("KNEE"),  _T("KNEW"),  _T("KNIT"),  _T("KNOB"),
    _T("KNOT"),  _T("KNOW"),  _T("KOCH"),  _T("KONG"),  _T("KUDO"),  _T("KURD"),  _T("KURT"),  _T("KYLE"),
    _T("LACE"),  _T("LACK"),  _T("LACY"),  _T("LADY"),  _T("LAID"),  _T("LAIN"),  _T("LAIR"),  _T("LAKE"),
    _T("LAMB"),  _T("LAME"),  _T("LAND"),  _T("LANE"),  _T("LANG"),  _T("LARD"),  _T("LARK"),  _T("LASS"),
    _T("LAST"),  _T("LATE"),  _T("LAUD"),  _T("LAVA"),  _T("LAWN"),  _T("LAWS"),  _T("LAYS"),  _T("LEAD"),
    _T("LEAF"),  _T("LEAK"),  _T("LEAN"),  _T("LEAR"),  _T("LEEK"),  _T("LEER"),  _T("LEFT"),  _T("LEND"),
    _T("LENS"),  _T("LENT"),  _T("LEON"),  _T("LESK"),  _T("LESS"),  _T("LEST"),  _T("LETS"),  _T("LIAR"),
    _T("LICE"),  _T("LICK"),  _T("LIED"),  _T("LIEN"),  _T("LIES"),  _T("LIEU"),  _T("LIFE"),  _T("LIFT"),
    _T("LIKE"),  _T("LILA"),  _T("LILT"),  _T("LILY"),  _T("LIMA"),  _T("LIMB"),  _T("LIME"),  _T("LIND"),
    _T("LINE"),  _T("LINK"),  _T("LINT"),  _T("LION"),  _T("LISA"),  _T("LIST"),  _T("LIVE"),  _T("LOAD"),
    _T("LOAF"),  _T("LOAM"),  _T("LOAN"),  _T("LOCK"),  _T("LOFT"),  _T("LOGE"),  _T("LOIS"),  _T("LOLA"),
    _T("LONE"),  _T("LONG"),  _T("LOOK"),  _T("LOON"),  _T("LOOT"),  _T("LORD"),  _T("LORE"),  _T("LOSE"),
    _T("LOSS"),  _T("LOST"),  _T("LOUD"),  _T("LOVE"),  _T("LOWE"),  _T("LUCK"),  _T("LUCY"),  _T("LUGE"),
    _T("LUKE"),  _T("LULU"),  _T("LUND"),  _T("LUNG"),  _T("LURA"),  _T("LURE"),  _T("LURK"),  _T("LUSH"),
    _T("LUST"),  _T("LYLE"),  _T("LYNN"),  _T("LYON"),  _T("LYRA"),  _T("MACE"),  _T("MADE"),  _T("MAGI"),
    _T("MAID"),  _T("MAIL"),  _T("MAIN"),  _T("MAKE"),  _T("MALE"),  _T("MALI"),  _T("MALL"),  _T("MALT"),
    _T("MANA"),  _T("MANN"),  _T("MANY"),  _T("MARC"),  _T("MARE"),  _T("MARK"),  _T("MARS"),  _T("MART"),
    _T("MARY"),  _T("MASH"),  _T("MASK"),  _T("MASS"),  _T("MAST"),  _T("MATE"),  _T("MATH"),  _T("MAUL"),
    _T("MAYO"),  _T("MEAD"),  _T("MEAL"),  _T("MEAN"),  _T("MEAT"),  _T("MEEK"),  _T("MEET"),  _T("MELD"),
    _T("MELT"),  _T("MEMO"),  _T("MEND"),  _T("MENU"),  _T("MERT"),  _T("MESH"),  _T("MESS"),  _T("MICE"),
    _T("MIKE"),  _T("MILD"),  _T("MILE"),  _T("MILK"),  _T("MILL"),  _T("MILT"),  _T("MIMI"),  _T("MIND"),
    _T("MINE"),  _T("MINI"),  _T("MINK"),  _T("MINT"),  _T("MIRE"),  _T("MISS"),  _T("MIST"),  _T("MITE"),
    _T("MITT"),  _T("MOAN"),  _T("MOAT"),  _T("MOCK"),  _T("MODE"),  _T("MOLD"),  _T("MOLE"),  _T("MOLL"),
    _T("MOLT"),  _T("MONA"),  _T("MONK"),  _T("MONT"),  _T("MOOD"),  _T("MOON"),  _T("MOOR"),  _T("MOOT"),
    _T("MORE"),  _T("MORN"),  _T("MORT"),  _T("MOSS"),  _T("MOST"),  _T("MOTH"),  _T("MOVE"),  _T("MUCH"),
    _T("MUCK"),  _T("MUDD"),  _T("MUFF"),  _T("MULE"),  _T("MULL"),  _T("MURK"),  _T("MUSH"),  _T("MUST"),
    _T("MUTE"),  _T("MUTT"),  _T("MYRA"),  _T("MYTH"),  _T("NAGY"),  _T("NAIL"),  _T("NAIR"),  _T("NAME"),
    _T("NARY"),  _T("NASH"),  _T("NAVE"),  _T("NAVY"),  _T("NEAL"),  _T("NEAR"),  _T("NEAT"),  _T("NECK"),
    _T("NEED"),  _T("NEIL"),  _T("NELL"),  _T("NEON"),  _T("NERO"),  _T("NESS"),  _T("NEST"),  _T("NEWS"),
    _T("NEWT"),  _T("NIBS"),  _T("NICE"),  _T("NICK"),  _T("NILE"),  _T("NINA"),  _T("NINE"),  _T("NOAH"),
    _T("NODE"),  _T("NOEL"),  _T("NOLL"),  _T("NONE"),  _T("NOOK"),  _T("NOON"),  _T("NORM"),  _T("NOSE"),
    _T("NOTE"),  _T("NOUN"),  _T("NOVA"),  _T("NUDE"),  _T("NULL"),  _T("NUMB"),  _T("OATH"),  _T("OBEY"),
    _T("OBOE"),  _T("ODIN"),  _T("OHIO"),  _T("OILY"),  _T("OINT"),  _T("OKAY"),  _T("OLAF"),  _T("OLDY"),
    _T("OLGA"),  _T("OLIN"),  _T("OMAN"),  _T("OMEN"),  _T("OMIT"),  _T("ONCE"),  _T("ONES"),  _T("ONLY"),
    _T("ONTO"),  _T("ONUS"),  _T("ORAL"),  _T("ORGY"),  _T("OSLO"),  _T("OTIS"),  _T("OTTO"),  _T("OUCH"),
    _T("OUST"),  _T("OUTS"),  _T("OVAL"),  _T("OVEN"),  _T("OVER"),  _T("OWLY"),  _T("OWNS"),  _T("QUAD"),
    _T("QUIT"),  _T("QUOD"),  _T("RACE"),  _T("RACK"),  _T("RACY"),  _T("RAFT"),  _T("RAGE"),  _T("RAID"),
    _T("RAIL"),  _T("RAIN"),  _T("RAKE"),  _T("RANK"),  _T("RANT"),  _T("RARE"),  _T("RASH"),  _T("RATE"),
    _T("RAVE"),  _T("RAYS"),  _T("READ"),  _T("REAL"),  _T("REAM"),  _T("REAR"),  _T("RECK"),  _T("REED"),
    _T("REEF"),  _T("REEK"),  _T("REEL"),  _T("REID"),  _T("REIN"),  _T("RENA"),  _T("REND"),  _T("RENT"),
    _T("REST"),  _T("RICE"),  _T("RICH"),  _T("RICK"),  _T("RIDE"),  _T("RIFT"),  _T("RILL"),  _T("RIME"),
    _T("RING"),  _T("RINK"),  _T("RISE"),  _T("RISK"),  _T("RITE"),  _T("ROAD"),  _T("ROAM"),  _T("ROAR"),
    _T("ROBE"),  _T("ROCK"),  _T("RODE"),  _T("ROIL"),  _T("ROLL"),  _T("ROME"),  _T("ROOD"),  _T("ROOF"),
    _T("ROOK"),  _T("ROOM"),  _T("ROOT"),  _T("ROSA"),  _T("ROSE"),  _T("ROSS"),  _T("ROSY"),  _T("ROTH"),
    _T("ROUT"),  _T("ROVE"),  _T("ROWE"),  _T("ROWS"),  _T("RUBE"),  _T("RUBY"),  _T("RUDE"),  _T("RUDY"),
    _T("RUIN"),  _T("RULE"),  _T("RUNG"),  _T("RUNS"),  _T("RUNT"),  _T("RUSE"),  _T("RUSH"),  _T("RUSK"),
    _T("RUSS"),  _T("RUST"),  _T("RUTH"),  _T("SACK"),  _T("SAFE"),  _T("SAGE"),  _T("SAID"),  _T("SAIL"),
    _T("SALE"),  _T("SALK"),  _T("SALT"),  _T("SAME"),  _T("SAND"),  _T("SANE"),  _T("SANG"),  _T("SANK"),
    _T("SARA"),  _T("SAUL"),  _T("SAVE"),  _T("SAYS"),  _T("SCAN"),  _T("SCAR"),  _T("SCAT"),  _T("SCOT"),
    _T("SEAL"),  _T("SEAM"),  _T("SEAR"),  _T("SEAT"),  _T("SEED"),  _T("SEEK"),  _T("SEEM"),  _T("SEEN"),
    _T("SEES"),  _T("SELF"),  _T("SELL"),  _T("SEND"),  _T("SENT"),  _T("SETS"),  _T("SEWN"),  _T("SHAG"),
    _T("SHAM"),  _T("SHAW"),  _T("SHAY"),  _T("SHED"),  _T("SHIM"),  _T("SHIN"),  _T("SHOD"),  _T("SHOE"),
    _T("SHOT"),  _T("SHOW"),  _T("SHUN"),  _T("SHUT"),  _T("SICK"),  _T("SIDE"),  _T("SIFT"),  _T("SIGH"),
    _T("SIGN"),  _T("SILK"),  _T("SILL"),  _T("SILO"),  _T("SILT"),  _T("SINE"),  _T("SING"),  _T("SINK"),
    _T("SIRE"),  _T("SITE"),  _T("SITS"),  _T("SITU"),  _T("SKAT"),  _T("SKEW"),  _T("SKID"),  _T("SKIM"),
    _T("SKIN"),  _T("SKIT"),  _T("SLAB"),  _T("SLAM"),  _T("SLAT"),  _T("SLAY"),  _T("SLED"),  _T("SLEW"),
    _T("SLID"),  _T("SLIM"),  _T("SLIT"),  _T("SLOB"),  _T("SLOG"),  _T("SLOT"),  _T("SLOW"),  _T("SLUG"),
    _T("SLUM"),  _T("SLUR"),  _T("SMOG"),  _T("SMUG"),  _T("SNAG"),  _T("SNOB"),  _T("SNOW"),  _T("SNUB"),
    _T("SNUG"),  _T("SOAK"),  _T("SOAR"),  _T("SOCK"),  _T("SODA"),  _T("SOFA"),  _T("SOFT"),  _T("SOIL"),
    _T("SOLD"),  _T("SOME"),  _T("SONG"),  _T("SOON"),  _T("SOOT"),  _T("SORE"),  _T("SORT"),  _T("SOUL"),
    _T("SOUR"),  _T("SOWN"),  _T("STAB"),  _T("STAG"),  _T("STAN"),  _T("STAR"),  _T("STAY"),  _T("STEM"),
    _T("STEW"),  _T("STIR"),  _T("STOW"),  _T("STUB"),  _T("STUN"),  _T("SUCH"),  _T("SUDS"),  _T("SUIT"),
    _T("SULK"),  _T("SUMS"),  _T("SUNG"),  _T("SUNK"),  _T("SURE"),  _T("SURF"),  _T("SWAB"),  _T("SWAG"),
    _T("SWAM"),  _T("SWAN"),  _T("SWAT"),  _T("SWAY"),  _T("SWIM"),  _T("SWUM"),  _T("TACK"),  _T("TACT"),
    _T("TAIL"),  _T("TAKE"),  _T("TALE"),  _T("TALK"),  _T("TALL"),  _T("TANK"),  _T("TASK"),  _T("TATE"),
    _T("TAUT"),  _T("TEAL"),  _T("TEAM"),  _T("TEAR"),  _T("TECH"),  _T("TEEM"),  _T("TEEN"),  _T("TEET"),
    _T("TELL"),  _T("TEND"),  _T("TENT"),  _T("TERM"),  _T("TERN"),  _T("TESS"),  _T("TEST"),  _T("THAN"),
    _T("THAT"),  _T("THEE"),  _T("THEM"),  _T("THEN"),  _T("THEY"),  _T("THIN"),  _T("THIS"),  _T("THUD"),
    _T("THUG"),  _T("TICK"),  _T("TIDE"),  _T("TIDY"),  _T("TIED"),  _T("TIER"),  _T("TILE"),  _T("TILL"),
    _T("TILT"),  _T("TIME"),  _T("TINA"),  _T("TINE"),  _T("TINT"),  _T("TINY"),  _T("TIRE"),  _T("TOAD"),
    _T("TOGO"),  _T("TOIL"),  _T("TOLD"),  _T("TOLL"),  _T("TONE"),  _T("TONG"),  _T("TONY"),  _T("TOOK"),
    _T("TOOL"),  _T("TOOT"),  _T("TORE"),  _T("TORN"),  _T("TOTE"),  _T("TOUR"),  _T("TOUT"),  _T("TOWN"),
    _T("TRAG"),  _T("TRAM"),  _T("TRAY"),  _T("TREE"),  _T("TREK"),  _T("TRIG"),  _T("TRIM"),  _T("TRIO"),
    _T("TROD"),  _T("TROT"),  _T("TROY"),  _T("TRUE"),  _T("TUBA"),  _T("TUBE"),  _T("TUCK"),  _T("TUFT"),
    _T("TUNA"),  _T("TUNE"),  _T("TUNG"),  _T("TURF"),  _T("TURN"),  _T("TUSK"),  _T("TWIG"),  _T("TWIN"),
    _T("TWIT"),  _T("ULAN"),  _T("UNIT"),  _T("URGE"),  _T("USED"),  _T("USER"),  _T("USES"),  _T("UTAH"),
    _T("VAIL"),  _T("VAIN"),  _T("VALE"),  _T("VARY"),  _T("VASE"),  _T("VAST"),  _T("VEAL"),  _T("VEDA"),
    _T("VEIL"),  _T("VEIN"),  _T("VEND"),  _T("VENT"),  _T("VERB"),  _T("VERY"),  _T("VETO"),  _T("VICE"),
    _T("VIEW"),  _T("VINE"),  _T("VISE"),  _T("VOID"),  _T("VOLT"),  _T("VOTE"),  _T("WACK"),  _T("WADE"),
    _T("WAGE"),  _T("WAIL"),  _T("WAIT"),  _T("WAKE"),  _T("WALE"),  _T("WALK"),  _T("WALL"),  _T("WALT"),
    _T("WAND"),  _T("WANE"),  _T("WANG"),  _T("WANT"),  _T("WARD"),  _T("WARM"),  _T("WARN"),  _T("WART"),
    _T("WASH"),  _T("WAST"),  _T("WATS"),  _T("WATT"),  _T("WAVE"),  _T("WAVY"),  _T("WAYS"),  _T("WEAK"),
    _T("WEAL"),  _T("WEAN"),  _T("WEAR"),  _T("WEED"),  _T("WEEK"),  _T("WEIR"),  _T("WELD"),  _T("WELL"),
    _T("WELT"),  _T("WENT"),  _T("WERE"),  _T("WERT"),  _T("WEST"),  _T("WHAM"),  _T("WHAT"),  _T("WHEE"),
    _T("WHEN"),  _T("WHET"),  _T("WHOA"),  _T("WHOM"),  _T("WICK"),  _T("WIFE"),  _T("WILD"),  _T("WILL"),
    _T("WIND"),  _T("WINE"),  _T("WING"),  _T("WINK"),  _T("WINO"),  _T("WIRE"),  _T("WISE"),  _T("WISH"),
    _T("WITH"),  _T("WOLF"),  _T("WONT"),  _T("WOOD"),  _T("WOOL"),  _T("WORD"),  _T("WORE"),  _T("WORK"),
    _T("WORM"),  _T("WORN"),  _T("WOVE"),  _T("WRIT"),  _T("WYNN"),  _T("YALE"),  _T("YANG"),  _T("YANK"),
    _T("YARD"),  _T("YARN"),  _T("YAWL"),  _T("YAWN"),  _T("YEAH"),  _T("YEAR"),  _T("YELL"),  _T("YOGA"),
    _T("YOKE")
};

TCHAR** COTPDictionary::words_ = words;



//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

COTPDictionary::COTPDictionary()
{

}

COTPDictionary::~COTPDictionary()
{

}

//////////////////////////////////////////////////////////////////////
// Get a dictionary word
//////////////////////////////////////////////////////////////////////

TCHAR* COTPDictionary::GetWord(int word)
{
	int index = word;

	if(index >= DICTIONARY_SIZE) {
		TRACE2( "Dictionary word requested (%d) was greater than dictionary size (%d)", index, DICTIONARY_SIZE );
		index = index % DICTIONARY_SIZE;
	}

	return words_[index];
}

