////////////////////////////////////////////////////////////////////////
//                                                                    //
// MicrOTP: A tool for computing one time passwords for S/Key logins. //
//                                                                    //
// (C) 2008 Tom Bramer                                                //
//                                                                    //
// This program is free software; you can redistribute it and/or      //
// modify it under the terms of the GNU General Public License        //
// as published by the Free Software Foundation; either version 2     //
// of the License, or (at your option) any later version.             //
//                                                                    //
// This program is distributed in the hope that it will be useful,    //
// but WITHOUT ANY WARRANTY; without even the implied warranty of     //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      //
// GNU General Public License for more details.                       //
//                                                                    //
// You should have received a copy of the GNU General Public License  //
// along with this program; if not, write to the Free Software        //
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA      //
// 02110-1301, USA.                                                   //
//                                                                    //
////////////////////////////////////////////////////////////////////////

// OTPGenerator.cpp: implementation of the COTPGenerator class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "otp3.h"
#include "OTPGenerator.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif



void CGenFold::Fold(DWORD* dwBuffer, int size)
{
	if(size == 4 || size == 5) {
		dwBuffer[0] ^= dwBuffer[2];
		dwBuffer[1] ^= dwBuffer[3];
		if(size == 5)
			dwBuffer[0] ^= dwBuffer[4];
	}
	else {
		TRACE1("Buffer to fold has more than 4 elements (%d)", size);
	}
}

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

COTPGenerator::COTPGenerator()
{

}

COTPGenerator::~COTPGenerator()
{

}

void COTPGenerator::ResultToWords(const std::vector<DWORD>& dwres, std::vector<int>& result)
{
	int par = 0;
	int bitcount = dwres.size() * sizeof(DWORD) * 8;
	std::bitset<96> bs(bitcount);

	for(std::vector<DWORD>::const_iterator vi = dwres.begin(); vi != dwres.end(); vi++)
	{
		for(DWORD i = 0, dw = *vi; i < sizeof(DWORD); i++, dw >>= 8) {
			bs <<= 8;
			bs |= dw & 0xFF;
		}
	}

	int i;

    for(i = 0; i < dwres.size(); ++i) {
		for(int j = 0; j < 32; j += 2) {
			par += (dwres[i] >> j) & 0x3;
		}
    }

	for(i = bitcount; i > 0; i -= 11)
	{
		int accu = 0;
		for(int j = 1; j <= 11; j++) {
			if(i - j >= 0) {
				accu = (accu << 1) | (bs.test(i - j) ? 1 : 0);
			}
			else {
				accu <<= 1;
			}
		}

		if(i < 11) {
			accu |= (par & 0x03);
		}

		result.push_back(accu);
	}

    return;
}

