////////////////////////////////////////////////////////////////////////
//                                                                    //
// MicrOTP: A tool for computing one time passwords for S/Key logins. //
//                                                                    //
// (C) 2008 Tom Bramer                                                //
//                                                                    //
// This program is free software; you can redistribute it and/or      //
// modify it under the terms of the GNU General Public License        //
// as published by the Free Software Foundation; either version 2     //
// of the License, or (at your option) any later version.             //
//                                                                    //
// This program is distributed in the hope that it will be useful,    //
// but WITHOUT ANY WARRANTY; without even the implied warranty of     //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      //
// GNU General Public License for more details.                       //
//                                                                    //
// You should have received a copy of the GNU General Public License  //
// along with this program; if not, write to the Free Software        //
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA      //
// 02110-1301, USA.                                                   //
//                                                                    //
////////////////////////////////////////////////////////////////////////

// OTPGenerator.h: interface for the COTPGenerator class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_OTPGENERATOR_H__A0ACEE98_013A_4833_B90E_2812D220B3FD__INCLUDED_)
#define AFX_OTPGENERATOR_H__A0ACEE98_013A_4833_B90E_2812D220B3FD__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class COTPGenerator  
{
public:
	virtual void GenOTP(int iter, std::string& seed, std::string& secret, std::vector<int>& result) = 0;
	virtual void ResultToWords(const std::vector<DWORD>& dwres, std::vector<int>& result);
	virtual ~COTPGenerator();

protected:
	COTPGenerator();

};

class CGenFold
{
public:
	void Fold(DWORD* dwBuffer, int size);
};


template <typename AlgorithmTraits>
class CGenOTPGenerator : public COTPGenerator
{
public:
	CGenOTPGenerator() { }
	virtual void GenOTP(int iter, std::string& seed, std::string& secret, std::vector<int>& result);

};

// Helper function
template <typename StringType>
StringType Lowercase(const StringType& var)
{
	std::string ret = var;
	for(std::string::iterator si = ret.begin(); si != ret.end(); si++) {
		*si = tolower(*si);
	}

	return ret;
}

// Dummy parameter pAlg is required here for algorithm because of Microsoft C++ compiler bug.
template <typename Algorithm, typename FoldAlgorithm, int BufferSize, int FoldSize>
void ComputeOTP(std::string& iv, int iter, std::vector<DWORD>& result, Algorithm* pAlg = NULL)
{
	Byte buffer[BufferSize];
	DWord* dwBuffer = (DWord*)buffer;
	Algorithm alg;
	FoldAlgorithm fa;
	
	alg.Initialize();
	alg.DoStep(reinterpret_cast<unsigned char*>(const_cast<char*>(iv.c_str())), iv.size());
	alg.Finalize();

	alg.GetByteResult(buffer, BufferSize);
	fa.Fold(dwBuffer, BufferSize / sizeof(DWORD));
	
	for(int i = iter; i > 0; i--) {
		alg.Initialize();
		alg.DoStep(buffer, FoldSize);
		alg.Finalize();
		alg.GetByteResult(buffer, BufferSize);
		fa.Fold(dwBuffer, BufferSize / sizeof(DWORD));
	}

	for(int j = 0; j < FoldSize / sizeof(DWORD); j++) {
		result.push_back(dwBuffer[j]);
	}
	
	return;
}

template <typename AlgorithmTraits>
void CGenOTPGenerator<AlgorithmTraits>::GenOTP(int iter, std::string& seed, std::string& secret, std::vector<int>& result)
{
	std::string iv = Lowercase(seed) + secret;
	std::vector<DWORD> dwres;
	ComputeOTP<CHashLibObject< AlgorithmTraits::AlgorithmType >, 
		       AlgorithmTraits::FoldType, 
			   AlgorithmTraits::BufferSize, 
			   AlgorithmTraits::FoldSize>(iv, iter, dwres);

	std::fill(iv.begin(), iv.end(), ' ');

	ResultToWords(dwres, result);
}



#endif // !defined(AFX_OTPGENERATOR_H__A0ACEE98_013A_4833_B90E_2812D220B3FD__INCLUDED_)
