////////////////////////////////////////////////////////////////////////
//                                                                    //
// MicrOTP: A tool for computing one time passwords for S/Key logins. //
//                                                                    //
// (C) 2008 Tom Bramer                                                //
//                                                                    //
// This program is free software; you can redistribute it and/or      //
// modify it under the terms of the GNU General Public License        //
// as published by the Free Software Foundation; either version 2     //
// of the License, or (at your option) any later version.             //
//                                                                    //
// This program is distributed in the hope that it will be useful,    //
// but WITHOUT ANY WARRANTY; without even the implied warranty of     //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      //
// GNU General Public License for more details.                       //
//                                                                    //
// You should have received a copy of the GNU General Public License  //
// along with this program; if not, write to the Free Software        //
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA      //
// 02110-1301, USA.                                                   //
//                                                                    //
////////////////////////////////////////////////////////////////////////

#ifdef HAVE_CONFIG_H
	#include <fv/config.h>
#endif

#include "stdafx.h"

#include <cassert>
#include "common.hpp"


CHashLibObjectRoot::CHashLibObjectRoot() : ref_(0)
{

}

bool CHashLibObjectRoot::InternalQueryInterface(unsigned long iid, void **pUnk)
{
	if(pUnk == NULL) return false;
	*pUnk = HasInterface(iid);

	if(*pUnk == NULL)
		return false;

	InternalAddRef();
	return true;
}

long CHashLibObjectRoot::InternalAddRef()
{
	ref_++;
	return ref_;
}

long CHashLibObjectRoot::InternalRelease()
{
	ref_--;
	if(ref_ == 0) {
		delete this;
		return 0;
	}

	return ref_;
}

CHashLibObjectRoot::~CHashLibObjectRoot()
{
}


void CHashAlgEnumerator::Init(const char** aStrAlgorithms, int ncaAlgorithms)
{
	nCurrent = -1;
	aStrAlgorithms_ = aStrAlgorithms;
	ncaAlgorithms_ = ncaAlgorithms;
}

const char * CHashAlgEnumerator::Current()
{
	assert(nCurrent >= 0 && nCurrent < ncaAlgorithms_);
	return aStrAlgorithms_[nCurrent];
}

bool CHashAlgEnumerator::MoveNext()
{
	nCurrent++;
	return nCurrent < ncaAlgorithms_;
}

void CHashAlgEnumerator::Reset()
{
	nCurrent = -1;
}

CHashAlgEnumerator::~CHashAlgEnumerator()
{
}

CHashLibClassFactory::CHashLibClassFactory(const char* strLibName, const char* strLibAuthor, const char* strAdditional, 
					                       int nLibVersion, const char** aStrAlgorithms, int ncaAlgorithms) 
: strLibName_(strLibName), strLibAuthor_(strLibAuthor), strAdditional_(strAdditional), nLibVersion_(nLibVersion),
  aStrAlgorithms_(aStrAlgorithms), ncaAlgorithms_(ncaAlgorithms)
{

}

bool CHashLibClassFactory::CreateInstance(const char *strName, IHashAlgorithm **pAlg)
{
	// This function should be implemented by the library on a case by case basis
	return false;
}

const char * CHashLibClassFactory::GetAdditionalInfo()
{
	return strAdditional_;
}

void CHashLibClassFactory::GetAlgEnumerator(IHashAlgEnumerator * *pEnum)
{
	CHashLibObject<CHashAlgEnumerator>* pEnumImpl = new CHashLibObject<CHashAlgEnumerator>();
	pEnumImpl->Init(aStrAlgorithms_, ncaAlgorithms_);
	pEnumImpl->QueryInterface(IID_IHashAlgEnumerator, reinterpret_cast<void**>(pEnum));
}

const char * CHashLibClassFactory::GetLibAuthor()
{
	return strLibAuthor_;
}

const char * CHashLibClassFactory::GetLibName()
{
	return strLibName_;
}


int CHashLibClassFactory::GetLibVersion()
{
	return nLibVersion_;
}

CHashLibClassFactory::~CHashLibClassFactory()
{
}

