////////////////////////////////////////////////////////////////////////
//                                                                    //
// MicrOTP: A tool for computing one time passwords for S/Key logins. //
//                                                                    //
// (C) 2008 Tom Bramer                                                //
//                                                                    //
// This program is free software; you can redistribute it and/or      //
// modify it under the terms of the GNU General Public License        //
// as published by the Free Software Foundation; either version 2     //
// of the License, or (at your option) any later version.             //
//                                                                    //
// This program is distributed in the hope that it will be useful,    //
// but WITHOUT ANY WARRANTY; without even the implied warranty of     //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      //
// GNU General Public License for more details.                       //
//                                                                    //
// You should have received a copy of the GNU General Public License  //
// along with this program; if not, write to the Free Software        //
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA      //
// 02110-1301, USA.                                                   //
//                                                                    //
////////////////////////////////////////////////////////////////////////

#ifndef __HASHLIB_COMMON_HPP__
#define __HASHLIB_COMMON_HPP__

#include <fv/types.hpp>
#include <fv/defs.hpp>
#include <fv/hashiface.hpp>

USING_NAMESPACE(PU::FV);

#define BEGIN_INTERFACE_MAPPING()                         \
virtual void* HasInterface(unsigned long riid)            \
{

#define MAP_INTERFACE(iid, itype)                         \
    if(iid == riid)                                       \
		return reinterpret_cast<void*>(static_cast<itype*>(this));

#define END_INTERFACE_MAPPING()                           \
	                                                      \
	return NULL;                                          \
}


template <class Base>
class CHashLibObject : public Base
{
public:
	virtual bool QueryInterface(unsigned long iid, void **pUnk)
	{ return Base::InternalQueryInterface(iid, pUnk); }
	
	virtual long AddRef()
	{ return Base::InternalAddRef(); }

	virtual long Release()
	{ return Base::InternalRelease(); }
	
};

class CHashLibObjectRoot
{
protected:
	virtual bool InternalQueryInterface(unsigned long iid, void **pUnk);
	virtual long InternalAddRef();
	virtual long InternalRelease();

	virtual void* HasInterface(unsigned long riid) = 0;

	virtual ~CHashLibObjectRoot();

	CHashLibObjectRoot();

private:
	long ref_;
};

class CHashAlgorithm : public CHashLibObjectRoot, public IHashAlgorithm
{
protected:
	BEGIN_INTERFACE_MAPPING()
		MAP_INTERFACE(IID_IHashLibUnknown, IHashLibUnknown)
		MAP_INTERFACE(IID_IHashAlgorithm, IHashAlgorithm)
	END_INTERFACE_MAPPING()
};

class CHashAlgEnumerator : public CHashLibObjectRoot, public IHashAlgEnumerator
{
public:
	virtual const char * Current();
	virtual bool MoveNext();
	virtual void Reset();
	virtual ~CHashAlgEnumerator();

	void Init(const char** aStrAlgorithms, int ncaAlgorithms);

protected:
	BEGIN_INTERFACE_MAPPING()
		MAP_INTERFACE(IID_IHashLibUnknown, IHashLibUnknown)
		MAP_INTERFACE(IID_IHashAlgEnumerator, IHashAlgEnumerator)
	END_INTERFACE_MAPPING()

private:
	const char** aStrAlgorithms_;
	int ncaAlgorithms_;
	int nCurrent;
};

class CHashLibClassFactory : public CHashLibObjectRoot, public IHashLibClassFactory
{
public:
	virtual void GetAlgEnumerator(IHashAlgEnumerator * *pEnum);
	virtual bool CreateInstance(const char *strName, IHashAlgorithm **pAlg);
	virtual const char * GetLibName();
	virtual const char * GetAdditionalInfo();
	virtual int GetLibVersion();
	virtual const char * GetLibAuthor();
	virtual ~CHashLibClassFactory();

	CHashLibClassFactory(const char* strLibName, const char* strLibAuthor, const char* strAdditional, 
						 int nLibVersion, const char** aStrAlgorithms, int ncaAlgorithms);

protected:
	BEGIN_INTERFACE_MAPPING()
		MAP_INTERFACE(IID_IHashLibUnknown, IHashLibUnknown)
		MAP_INTERFACE(IID_IHashLibClassFactory, IHashLibClassFactory)
	END_INTERFACE_MAPPING()

private:
	const char* strLibName_;
	const char* strLibAuthor_;
	const char* strAdditional_;
	int nLibVersion_;
	const char** aStrAlgorithms_;
	int ncaAlgorithms_;
};


#define IMPLEMENT_HLM_MODULE(__class_factory__)               \
extern "C"                                                    \
{                                                             \
    __declspec(dllexport) IHashLibClassFactory* InitFactory() \
    {                                                         \
        IHashLibClassFactory* pFactory = NULL;                \
		CHashLibObject<__class_factory__>* pCF = new CHashLibObject<__class_factory__>();     \
		pCF->QueryInterface(IID_IHashLibClassFactory,         \
		     reinterpret_cast<void**>(&pFactory));            \
        return pFactory;                                      \
    }                                                         \
                                                              \
}

#endif /* __HASHLIB_COMMON_HPP__ */


