////////////////////////////////////////////////////////////////////////
//                                                                    //
// MicrOTP: A tool for computing one time passwords for S/Key logins. //
//                                                                    //
// (C) 2008 Tom Bramer                                                //
//                                                                    //
// This program is free software; you can redistribute it and/or      //
// modify it under the terms of the GNU General Public License        //
// as published by the Free Software Foundation; either version 2     //
// of the License, or (at your option) any later version.             //
//                                                                    //
// This program is distributed in the hope that it will be useful,    //
// but WITHOUT ANY WARRANTY; without even the implied warranty of     //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      //
// GNU General Public License for more details.                       //
//                                                                    //
// You should have received a copy of the GNU General Public License  //
// along with this program; if not, write to the Free Software        //
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA      //
// 02110-1301, USA.                                                   //
//                                                                    //
////////////////////////////////////////////////////////////////////////

#ifndef __HASHLIBIFACE_HPP__
#define __HASHLIBIFACE_HPP__

NAMESPACE_BEGIN(PU)
NAMESPACE_BEGIN(FV)

const unsigned long IID_IHashLibUnknown      = 0x10000;
const unsigned long IID_IHashAlgorithm       = 0x10001;
const unsigned long IID_IHashAlgEnumerator   = 0x10002;
const unsigned long IID_IHashLibClassFactory = 0x10003;


typedef unsigned char Byte;
typedef unsigned long BufferLength;

class IHashLibClassFactory;

typedef IHashLibClassFactory* (*FactoryProc)();
typedef void (*DestroyProc)();

class IHashLibUnknown
{
public:
	virtual bool QueryInterface(unsigned long iid, void** pUnk) = 0;
	virtual long AddRef() = 0;
	virtual long Release() = 0;
};

class IHashAlgorithm : public IHashLibUnknown {
public:
    virtual const char* GetAlgName() = 0;
    virtual const char* GetImpAuthor() = 0;
    virtual int GetImpVersion() = 0;
    virtual const char* GetAdditionalInfo() = 0;
    virtual void Initialize() = 0;
    virtual void DoStep(Byte* pBuffer, BufferLength nLength) = 0;
    virtual void Finalize() = 0;
    virtual int GetStringResult(char* pBuffer, int nBufferSize) = 0;
    virtual int GetByteResult(Byte* pBuffer, int nBufferSize) = 0;
};

class IHashAlgEnumerator : public IHashLibUnknown {
public:
	virtual const char* Current() = 0;
	virtual bool MoveNext() = 0;
	virtual void Reset() = 0;
};

class IHashLibClassFactory : public IHashLibUnknown {
public:
    virtual void GetAlgEnumerator(IHashAlgEnumerator** pEnum) = 0;
    virtual bool CreateInstance(const char* strName, IHashAlgorithm** pAlg) = 0;
	virtual const char* GetLibName() = 0;
    virtual const char* GetLibAuthor() = 0;
    virtual int GetLibVersion() = 0;
    virtual const char* GetAdditionalInfo() = 0;
};

inline void HashToHexStr(CxString& strResult, Byte* bp, unsigned int length)
{
    unsigned int n;
    const CxChar* cTable = _CxT("0123456789ABCDEF");
    strResult = _CxT("");
        
    for (n = 0; n < length; n++) {
        if(*bp < 0x10) {
            strResult += _CxT("0");
        }
        else {
            strResult += cTable[(*bp & 0xF0) >> 4];
        }
        strResult += cTable[(*bp++ & 0x0F)];
    }
}

inline void HashToHexStr(char* strResult, Byte* bp, unsigned int length)
{
    unsigned int n;
    const char* cTable = "0123456789ABCDEF";
    memset(strResult, 0, (length * 2) + 1);
        
    for (n = 0; n < length; n++) {
        if(*bp < 0x10) {
			*strResult++ = '0';
        }
        else {
			*strResult++ = cTable[(*bp & 0xF0) >> 4];
        }
        *strResult++ = cTable[(*bp++ & 0x0F)];
    }
}

NAMESPACE_END(FV)
NAMESPACE_END(PU)

#endif
