////////////////////////////////////////////////////////////////////////
//                                                                    //
// MicrOTP: A tool for computing one time passwords for S/Key logins. //
//                                                                    //
// (C) 2008 Tom Bramer                                                //
//                                                                    //
// This program is free software; you can redistribute it and/or      //
// modify it under the terms of the GNU General Public License        //
// as published by the Free Software Foundation; either version 2     //
// of the License, or (at your option) any later version.             //
//                                                                    //
// This program is distributed in the hope that it will be useful,    //
// but WITHOUT ANY WARRANTY; without even the implied warranty of     //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      //
// GNU General Public License for more details.                       //
//                                                                    //
// You should have received a copy of the GNU General Public License  //
// along with this program; if not, write to the Free Software        //
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA      //
// 02110-1301, USA.                                                   //
//                                                                    //
////////////////////////////////////////////////////////////////////////

#ifndef __PU_TYPES_HPP__
#define __PU_TYPES_HPP__

#if defined(__WIN32__) || defined(_WIN32_WCE)

#include <windows.h>
#include <tchar.h>

#else

#if defined(UNICODE) || defined(_UNICODE)
typedef wchar_t TCHAR;
#else
typedef char TCHAR;
#endif

typedef char CHAR;
typedef wchar_t WCHAR;
typedef unsigned char BYTE;
typedef unsigned long DWORD;

#define VOID void
typedef VOID* PVOID;
typedef VOID* LPVOID;
typedef DWORD* PDWORD;
typedef DWORD* LPDWORD;
typedef BYTE* PBYTE;
typedef BYTE* LPBYTE;

typedef CHAR* LPSTR;
typedef const LPSTR LPCSTR;
typedef WCHAR* LPWSTR;
typedef const LPWSTR LPCWSTR;
typedef TCHAR* LPTSTR;
typedef const LPTSTR LPCTSTR;
#define CALLBACK 

#endif /* __WIN32__ */

#include <string>

// If QWORD is already defined by the preprocessor,
// undefine it so we don't conflict with our typedef
#ifdef QWORD
#undef QWORD
#endif

typedef TCHAR   CxChar;
typedef CHAR    CxCharA;
typedef WCHAR   CxCharW;
typedef BYTE    CxByte;

typedef std::basic_string<CxChar>        CxString;
typedef std::basic_string<CxCharA>       CxStringA;
typedef std::basic_string<CxCharW>       CxStringW;
typedef std::basic_string<CxByte>        CxBString;
typedef unsigned __int64                 QWORD;
typedef __int64                          QINT;


template <typename T>
class CxStringT : public std::basic_string<T>
{
public:
    CxStringT() : std::basic_string<T>() { }
    CxStringT(const CxStringT<T>& rhs) : std::basic_string<T>(rhs) { }
    CxStringT(const T* ptr) : std::basic_string<T>(ptr) { }
};

typedef DWORD (CALLBACK *ThreadProc)(LPVOID);
typedef VOID (CALLBACK *QueueAPCProc)(DWORD);

#define _CxT(s)    _T(s)
#define _CxChar(s) _TCHAR(s)

enum VerStatus {
    NotChecked = 0,
    ErrorChecking,
    Invalid,
    SumSizeValid,
    AllValid,
    NotCalculated
};

enum SortColumn {
    FileName = 0,
    FileSize,
    ModDate,
    Attrib,
    Algorithm,
    Hash,
    Verification,
    Unique,
    Sort,
    Checked
};


#endif // __FV_TYPES_HPP__
