////////////////////////////////////////////////////////////////////////
//                                                                    //
// MicrOTP: A tool for computing one time passwords for S/Key logins. //
//                                                                    //
// (C) 2008 Tom Bramer                                                //
//                                                                    //
// This program is free software; you can redistribute it and/or      //
// modify it under the terms of the GNU General Public License        //
// as published by the Free Software Foundation; either version 2     //
// of the License, or (at your option) any later version.             //
//                                                                    //
// This program is distributed in the hope that it will be useful,    //
// but WITHOUT ANY WARRANTY; without even the implied warranty of     //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      //
// GNU General Public License for more details.                       //
//                                                                    //
// You should have received a copy of the GNU General Public License  //
// along with this program; if not, write to the Free Software        //
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA      //
// 02110-1301, USA.                                                   //
//                                                                    //
////////////////////////////////////////////////////////////////////////

// otp3View.cpp : implementation of the COtp3View class
//

#include "stdafx.h"
#include "otp3.h"

#include "otp3Doc.h"
#include "otp3View.h"

#include "OTPGenerator.h"
#include "OTPDictionary.h"
#include "md.hpp"
#include "sha.hpp"
#include "rmd.hpp"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// COtp3View

IMPLEMENT_DYNCREATE(COtp3View, CFormView)

BEGIN_MESSAGE_MAP(COtp3View, CFormView)
	//{{AFX_MSG_MAP(COtp3View)
	ON_BN_CLICKED(IDC_CALC, OnCalc)
	ON_CBN_SELCHANGE(IDC_METHOD, OnSelchangeMethod)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// COtp3View construction/destruction

COtp3View::COtp3View()
	: CFormView(COtp3View::IDD)
{
	//{{AFX_DATA_INIT(COtp3View)
	challenge_ = _T("");
	method_ = -1;
	secret_ = _T("");
	result_ = _T("");
	//}}AFX_DATA_INIT
	// TODO: add construction code here

}

COtp3View::~COtp3View()
{
}

void COtp3View::DoDataExchange(CDataExchange* pDX)
{
	CFormView::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(COtp3View)
	DDX_Text(pDX, IDC_CHALLENGE, challenge_);
	DDX_CBIndex(pDX, IDC_METHOD, method_);
	DDX_Text(pDX, IDC_PASSWORD, secret_);
	DDX_Text(pDX, IDC_RESULT, result_);
	//}}AFX_DATA_MAP
}

BOOL COtp3View::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CFormView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// COtp3View diagnostics

#ifdef _DEBUG
void COtp3View::AssertValid() const
{
	CFormView::AssertValid();
}

void COtp3View::Dump(CDumpContext& dc) const
{
	CFormView::Dump(dc);
}

COtp3Doc* COtp3View::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(COtp3Doc)));
	return (COtp3Doc*)m_pDocument;
}
#endif //_DEBUG

template <typename CharT>
char Narrow(CharT in)
{
	return in & 0xFF;
}

template char Narrow<char>(char in);
template char Narrow<wchar_t>(wchar_t in);

template <typename Algorithm>
class AlgTraits
{
};

template <>
class AlgTraits<CMD4Algorithm>
{
public:
	enum { BufferSize = 16, FoldSize = 8 };
	
	typedef CMD4Algorithm AlgorithmType;
	typedef CGenFold FoldType;
};

template <>
class AlgTraits<CMD5Algorithm>
{
public:
	enum { BufferSize = 16, FoldSize = 8 };

	typedef CMD5Algorithm AlgorithmType;
	typedef CGenFold FoldType;
};

template <>
class AlgTraits<CSHA1Algorithm>
{
public:
	enum { BufferSize = 20, FoldSize = 8 };

	typedef CSHA1Algorithm AlgorithmType;
	typedef CGenFold FoldType;
};

template <>
class AlgTraits<CRMD160Algorithm>
{
public:
	enum { BufferSize = 20, FoldSize = 8 };
	
	typedef CRMD160Algorithm AlgorithmType;
	typedef CGenFold FoldType;
};




/////////////////////////////////////////////////////////////////////////////
// COtp3View message handlers

void COtp3View::OnCalc() 
{
	UpdateData(TRUE);

	COTPGenerator* gen = NULL;
	int iter = 0;
	CString seed;
	std::vector<int> result;
	int pos = challenge_.Find(_TCHAR(' '));
	if(pos >= 0) {
		iter = _ttoi(challenge_.Left(pos));
		seed = challenge_.Mid(pos + 1);
		seed.TrimLeft();
		seed.TrimRight();
	}
	else {
		AfxMessageBox(_T("Challenge must be in the format of [iter] <space> [seed]!"), MB_OK | MB_ICONEXCLAMATION, 0);
		return;
	}

	std::basic_string<TCHAR> cseed = seed;
	std::basic_string<TCHAR> csecret = secret_;
	std::string narrow_seed(cseed.size(), ' ');
	std::string narrow_secret(csecret.size(), ' ');

	for(int x = 0; x < secret_.GetLength(); x++) {
		secret_.SetAt(x, _TCHAR(' '));
	}

	std::transform(cseed.begin(), cseed.end(), narrow_seed.begin(), Narrow<TCHAR>);
	std::transform(csecret.begin(), csecret.end(), narrow_secret.begin(), Narrow<TCHAR>);

	std::fill(csecret.begin(), csecret.end(), _TCHAR(' '));

	
	TRACE1("Method selected: %d", method_);
	switch(method_) {
	case 0: gen = new CGenOTPGenerator< AlgTraits<CMD4Algorithm> >(); break;
	case 1: gen = new CGenOTPGenerator< AlgTraits<CMD5Algorithm> >(); break;
	case 2: gen = new CGenOTPGenerator< AlgTraits<CSHA1Algorithm> >(); break;
	case 3: gen = new CGenOTPGenerator< AlgTraits<CRMD160Algorithm> >(); break;
	}

	if(NULL == gen) {
		AfxMessageBox(_T("Unknown OTP generation method!"), MB_OK | MB_ICONERROR, 0);
		return;
	}

	gen->GenOTP(iter, narrow_seed, narrow_secret, result);

	std::fill(narrow_secret.begin(), narrow_secret.end(), ' ');

	delete gen;

	result_ = "";
	for(std::vector<int>::iterator vi = result.begin(); vi != result.end(); vi++)
	{
		result_ += COTPDictionary::GetWord(*vi);
		result_ += _T(" ");
	}

	iter -= 1;
	iter = (iter < 0 ? 0 : iter);
	challenge_.Format(_T("%d %s"), iter, seed);
	secret_ = _T("");
	theApp.WriteProfileString(_T("Settings"), _T("LastChallenge"), challenge_);


	UpdateData(FALSE);

}

void COtp3View::OnSelchangeMethod() 
{
	UpdateData(true);
	theApp.WriteProfileInt(_T("Settings"), _T("LastMethod"), method_);
}

BOOL COtp3View::Create(LPCTSTR lpszClassName, LPCTSTR lpszWindowName, DWORD dwStyle, const RECT& rect, CWnd* pParentWnd, UINT nID, CCreateContext* pContext) 
{	
	BOOL ret = CFormView::Create(lpszClassName, lpszWindowName, dwStyle, rect, pParentWnd, nID, pContext);
	
	// Initialize the form fields.
	method_ = theApp.GetProfileInt(_T("Settings"), _T("LastMethod"), 0);
	challenge_ = theApp.GetProfileString(_T("Settings"), _T("LastChallenge"), _T(""));
	UpdateData(false);

	return ret;
}
